# Simple
# ---------------
import os
import os,sys
import os, sys
import os as a, sys as b

from os import path
from os import path, getcwd
from os import path as p, getcwd as whereami

'import file'
'from blah import z'

# Harder
# ----------------
import _private_module
from _private_module import _private_name

from . import blah

import      os
from        os     import path
"""
import os
from os import path
"""

from os import (
    path,
    getcwd,
)

from os import (
    path,
    getcwd
)

import os, \
    sys

# Wtf
# ----------------

from os import (
    path,
    getcwd,
    getenv,)

from os import (
    path, # Lolz
# Yo

        # Yo
    getcwd    ,
    getenv,
)

from os import (
    path,
    getcwd,
    getenv,)

import \
    os

from\
    pathlib\
    import\
        Path

from path\
    import\
        lib